@@ -15,7 +15,7 @@ use SHARYANTO::Package::Util qw(package_exists);
 use Tie::Cache;
 use URI::Split qw(uri_split uri_join);
 
-our $VERSION = '0.52'; # VERSION
+our $VERSION = '0.53'; # VERSION
 
 our $re_perl_package =
     qr/\A[A-Za-z_][A-Za-z_0-9]*(::[A-Za-z_][A-Za-z_0-9]*)*\z/;
@@ -59,8 +59,7 @@ sub new {
     }
     $self->{_typeacts} = \%typeacts;
 
-    $self->{cache_size}            //= 100;
-    $self->{disk_cache}            //= 0;
+    $self->{cache_size}            //= 100; # for caching wrap result in memory
     $self->{use_tx}                //= 0;
     $self->{wrap}                  //= 1;
     $self->{custom_tx_manager}     //= undef;
@@ -93,14 +92,6 @@ sub new {
         }
     }
 
-    # to cache wrapped result
-    if ($self->{cache_size}) {
-        tie my(%cache), 'Tie::Cache', $self->{cache_size};
-        $self->{_cache} = \%cache;
-    } else {
-        $self->{_cache} = {};
-    }
-
     $self;
 }
 
@@ -195,8 +186,6 @@ sub _load_module {
     $module_p =~ s!::!/!g;
     $module_p .= ".pm";
 
-    $req->{-module_p} = $module_p; # for convenience of _get_cache_path
-
     # module has been required before and successfully loaded
     return if $INC{$module_p};
 
@@ -244,28 +233,6 @@ sub _load_module {
     return $res;
 }
 
-sub _get_cache_path {
-    my ($self, $name, $hash_source, $req) = @_;
-
-    require File::Spec;
-    my $dir = File::Spec->tmpdir;
-
-    require Data::Dumper;
-    require Digest::MD5;
-    my $file_md5;
-    {
-        local $/;
-        open my($fh), $INC{$req->{-module_p}} // $0;
-        $file_md5 = Digest::MD5::md5_hex(~~<$fh>);
-    }
-    my $hash = Digest::MD5::md5_hex(
-        Data::Dumper::Dumper($hash_source) . $file_md5);
-
-    $name =~ s/::/./g; # for windows
-    my $fname = "$name.$hash.wrapcache";
-    sprintf("%s/%s", $dir, $fname);
-}
-
 sub _get_code_and_meta {
     no strict 'refs';
     my ($self, $req) = @_;
@@ -296,7 +263,6 @@ sub _get_code_and_meta {
                        "module $req->{-perl_package}")
             unless defined &{$name};
 
-        my $cache_path;
         my $wrapres;
       GET_CODE:
         {
@@ -305,28 +271,6 @@ sub _get_code_and_meta {
                 last GET_CODE;
             }
 
-            goto NOT_CACHE if !$self->{disk_cache};
-
-            $cache_path = $self->_get_cache_path(
-                $name, [$self->{extra_wrapper_args},
-                        $self->{extra_wrapper_convert}], $req);
-
-            goto NOT_CACHE unless -f $cache_path;
-
-            # expire after 24h, for other factors that might make cache stale,
-            # like updated Perinci::Sub::Wrapper version, etc.
-            goto NOT_CACHE if (-M $cache_path) >= 1;
-
-            $log->tracef("Using wrap cache %s ...", $cache_path);
-            my $res;
-            $res = do $cache_path;
-            return err(500, "Can't load disk cache '$cache_path': $@")
-                if $@;
-            $code = $res->[0];
-            $meta = $res->[1];
-            last GET_CODE;
-
-          NOT_CACHE:
             require Perinci::Sub::Wrapper;
             $wrapres = Perinci::Sub::Wrapper::wrap_sub(
                 sub_name=>$name, meta=>$meta,
@@ -350,28 +294,6 @@ sub _get_code_and_meta {
 
         $self->{_cache}{$name} = [$code, $meta, $extra]
             if $self->{cache_size};
-        if ($wrapres && $cache_path) {
-            open my($fh), ">", $cache_path;
-            $self->{use_utf8} = 1; # TMP
-            binmode($fh, ":utf8") if $self->{use_utf8};
-            {
-                local $Data::Dumper::Deparse = 1;
-                local $Data::Dumper::Purity = 1;
-                local $Data::Dumper::Terse = 1;
-                local $Data::Dumper::Indent = 0;
-                print $fh join(
-                    "",
-                    ($self->{use_utf8} ? "use utf8;" : ""),
-                    "[do{",
-                    $wrapres->[2]{source},
-                    ", ",
-                    Data::Dumper::Dumper($meta),
-                    "}];\n",
-                );
-            }
-            close $fh;
-            chmod 0600, $cache_path;
-        }
     }
     unless (defined $meta->{entity_v}) {
         my $ver = ${ $req->{-perl_package} . "::VERSION" };
@@ -659,69 +581,19 @@ sub actionmeta_complete_arg_val { +{
 } }
 
 sub action_complete_arg_val {
+    require Perinci::Sub::Complete;
+
     my ($self, $req) = @_;
     my $arg = $req->{arg} or return err(400, "Please specify arg");
     my $word = $req->{word} // "";
+    my $ci = $req->{ci};
 
     my $res = $self->_get_code_and_meta($req);
     return $res unless $res->[0] == 200;
     my (undef, $meta) = @{$res->[2]};
-    my $args_p = $meta->{args} // {};
-    my $arg_p = $args_p->{$arg}
-        or return err(400, "Unknown function arg '$arg'");
-
-    my $words;
-    eval { # completion sub can die, etc.
-
-        if ($arg_p->{completion}) {
-            $words = $arg_p->{completion}(word=>$word);
-            die "Completion sub does not return array"
-                unless ref($words) eq 'ARRAY';
-            return;
-        }
-
-        my $sch = $arg_p->{schema};
-
-        my ($type, $cs) = @{$sch};
-        if ($cs->{in}) {
-            $words = $cs->{in};
-            return;
-        }
-
-        if ($type =~ /^int\*?$/) {
-            my $limit = 100;
-            if ($cs->{between} &&
-                    $cs->{between}[0] - $cs->{between}[0] <= $limit) {
-                $words = [$cs->{between}[0] .. $cs->{between}[1]];
-                return;
-            } elsif ($cs->{xbetween} &&
-                    $cs->{xbetween}[0] - $cs->{xbetween}[0] <= $limit) {
-                $words = [$cs->{xbetween}[0]+1 .. $cs->{xbetween}[1]-1];
-                return;
-            } elsif (defined($cs->{min}) && defined($cs->{max}) &&
-                         $cs->{max}-$cs->{min} <= $limit) {
-                $words = [$cs->{min} .. $cs->{max}];
-                return;
-            } elsif (defined($cs->{min}) && defined($cs->{xmax}) &&
-                         $cs->{xmax}-$cs->{min} <= $limit) {
-                $words = [$cs->{min} .. $cs->{xmax}-1];
-                return;
-            } elsif (defined($cs->{xmin}) && defined($cs->{max}) &&
-                         $cs->{max}-$cs->{xmin} <= $limit) {
-                $words = [$cs->{xmin}+1 .. $cs->{max}];
-                return;
-            } elsif (defined($cs->{xmin}) && defined($cs->{xmax}) &&
-                         $cs->{xmax}-$cs->{xmin} <= $limit) {
-                $words = [$cs->{min}+1 .. $cs->{max}-1];
-                return;
-            }
-        }
-
-        $words = [];
-    };
-    return err(500, "Completion died: $@") if $@;
-
-    [200, "OK", [grep /^\Q$word\E/, @$words]];
+    [200, "OK",
+     Perinci::Sub::Complete::complete_arg_val(meta=>$meta, word=>$word,
+                                              arg=>$arg, ci=>$ci)];
 }
 
 sub actionmeta_child_metas { +{
@@ -973,13 +845,15 @@ __END__
 
 =pod
 
+=encoding utf-8
+
 =head1 NAME
 
 Perinci::Access::Schemeless - Base class for Perinci::Access::Perl
 
 =head1 VERSION
 
-version 0.52
+version 0.53
 
 =head1 DESCRIPTION
 
@@ -1132,23 +1006,6 @@ need not involve wrapping again. Setting this to 0 disables caching.
 Caching is implemented inside C<get_meta()> and C<get_code()> so you might want
 to implement your own caching if you override those.
 
-See also: C<disk_cache>
-
-=item * disk_cache => BOOL (default: 0)
-
-Whether to cache function wrapping result to disk, to shave startup overhead for
-the next invocation of program. Useful for one-off programs like command-line
-scripts (see L<Perinci::CmdLine>) that get invoked often; not really useful for
-long-running programs (e.g. daemons).
-
-Currently wrapping result is cached in C<<
-$TMPDIR/Package::SubPackage::funcname.$HASH.wrapcache >> where C<$HASH> is
-calculated from C<extra_wrapper_args> and C<extra_wrapper_convert>.
-
-See also: C<cache_size>
-
-TODO: C<disk_cache_path>
-
 =item * allow_paths => REGEX|STR|ARRAY
 
 If defined, only requests with C<uri> matching specified path will be allowed.
@@ -1204,6 +1061,23 @@ the Riap request key 'uri', as there is no server in this case.
 
 L<Riap>, L<Rinci>
 
+=head1 HOMEPAGE
+
+Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Access-Perl>.
+
+=head1 SOURCE
+
+Source repository is at L<https://github.com/sharyanto/perl-Perinci-Access-Perl>.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website
+http://rt.cpan.org/Public/Dist/Display.html?Name=Perinci-Access-Perl
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 AUTHOR
 
 Steven Haryanto <stevenharyanto@gmail.com>
